#include <Wire.h> //I2C library
#include <I2C_eeprom.h>

I2C_eeprom flash(0x50, 32768);


#define MARKER_BEGIN  65
#define MARKER_NEXT_FILE  66
#define MARKER_ARDUINO_RESET  67

#define BUFFER_SIZE 16
byte buffer[BUFFER_SIZE];

byte max_files;
int last_free_address = 0;

void setup() 
{
    Serial.begin(9600);
    Serial.print("Starting test...\n");

    pinMode(8, INPUT);
}

byte getMaxStrings()
{
    // Ignores everything untill the start marker
    while (Serial.read() != MARKER_BEGIN);
//    digitalWrite(2, HIGH);
    
    // Once it receives the marker, the next byte should be the number of files
    while (Serial.available() == 0);
//    digitalWrite(3, HIGH);
    byte received = Serial.read();
    
    flash.writeByte(0, received);        

    
    // Writes it back to the computer to make sure things are ok
    Serial.write (flash.readByte(0));
    return received;
}

/*
unsigned int getNextFile(unsigned int flash_pos, byte file_number)
{
    // Ignores everything untill the start marker
    while (Serial.read() != MARKER_NEXT_FILE);

    // Writes the initial position of the file in the EEPROM
    flash.writeInt(1+2*file_number, flash_pos);
    
    // Once it receives the marker, the next bytes should be the file length
    while (Serial.available() <2);
    byte size_lo = Serial.read();
    byte size_hi = Serial.read();
    byte crc_byte = 0;
    int file_size = ( size_hi << 8 | size_lo );
    // Writes the size of the file to the EEPROM
    flash.writeInt(flash_pos, file_size);
    
    // Writes it back to the computer to make sure things are ok
    Serial.write (size_lo);
    Serial.write (size_hi);

    unsigned int bytes_remaining = file_size;
    unsigned int buffer_pos = 0;
    unsigned int write_address = 2 + flash_pos;
    //Receives all the bytes of the file
    while (bytes_remaining >0 )
    {
        buffer_pos = 0;
        while (bytes_remaining >0 && buffer_pos < BUFFER_SIZE)
        {
            while (Serial.available() == 0);
            buffer[buffer_pos] = Serial.read();
            crc_byte = crc_byte^buffer[buffer_pos];
            buffer_pos++;
            bytes_remaining--;
        }
        // Writes the file data to the EEPROM
        flash.writeBlock(write_address, buffer, buffer_pos);
        write_address += buffer_pos;
    }
    //Sends a simple byte CRC of the file
    Serial.write(crc_byte);
    digitalWrite(4, HIGH);
    return file_size;
}
*/

unsigned int getNextFile(unsigned int flash_pos, byte file_number)
{
    // Ignores everything untill the start marker
    while (Serial.read() != MARKER_NEXT_FILE);

    // Writes the initial position of the file in the EEPROM
    flash.writeInt(1+2*file_number, flash_pos);
//    Serial.print("Starting to write in position:"); Serial.println(flash.readInt(1+2*file_number));

    // Once it receives the marker, the next bytes should be the file length
    while (Serial.available() <2);
    byte size_lo = Serial.read();
    byte size_hi = Serial.read();
    byte crc_byte = 0;
    int file_size = ( size_hi << 8 | size_lo );

    Serial.print("File size received:");
    Serial.println(file_size);

    // Writes the size of the file to the EEPROM
    flash.writeInt(flash_pos, file_size);

    Serial.print("File size written::");
    Serial.println(flash.readInt(flash_pos));
    
    unsigned int bytes_remaining = file_size;
    unsigned int buffer_pos = 0;
    unsigned int write_address = 2 + flash_pos;
    //Receives all the bytes of the file
    while (bytes_remaining >0 )
    {
        buffer_pos = 0;
        while (bytes_remaining >0 && buffer_pos < BUFFER_SIZE)
        {
            while (Serial.available() == 0);
            buffer[buffer_pos] = Serial.read();
            crc_byte = crc_byte^buffer[buffer_pos];
            buffer_pos++;
            bytes_remaining--;
        }
        // Writes the file data to the EEPROM
        flash.writeBlock(write_address, buffer, buffer_pos);
        write_address += buffer_pos;
    }
    //Sends a simple byte CRC of the file
    Serial.print("File received, CRC:");
    if (crc_byte < 0x10) Serial.print('0');
    Serial.println(crc_byte, HEX);
    return file_size;
}


void loop()
{    
// Used for debugging    
//    if (!digitalRead(8))
    if (true)
    {
        // Tells the pc it's ready to receive stuff
        Serial.write(MARKER_ARDUINO_RESET);
        max_files = getMaxStrings();
        // First byte is the number of files stored 
        last_free_address += 1;
        // Next bytes are the beginning addresses of each file
        // So we need to reserve max_files * 2 bytes
        last_free_address += max_files * 2;
        
        for (byte current_file=0; current_file<max_files; current_file++)
        {
            last_free_address += 2 + getNextFile(last_free_address, current_file);
        }   
    }
    else
    {
        flash.dumpEEPROM(0, 100);
        flash.dumpEEPROM(2660, 100);
        
    } 
    while(1);
}
