import processing.serial.*;

Serial myPort;

//String filenames[] = {"teste1.txt", "teste2.txt", "teste3.txt"};
//String filenames[] = { "switch_100.gif", "clock_100.gif", "house_100.gif", "plant.gif", "cancel_50.gif", "add_50.gif", "save_50.gif", "on_50.gif", "off_50.gif",  "window_open_50.gif", "window_closed_50.gif", "key_50.gif", "lily.gif"};
String filenames[] = { "ardomotics_icon_100.gif", 
                       "switch_100.gif", 
                       "clock_100.gif", 
                       "house_100.gif",
                       "power_blue.gif",
                       "blueprint.gif", 
                       "cancel_50.gif",
                       "add_50.gif", 
                       "save_50.gif", 
                       "on_50.gif",
                       "off_50.gif",  
                       "window_open_50.gif",
                       "window_closed_50.gif", 
                       "key_on_50.gif",
                       "key_off_50.gif",
                       "wall_outlet_on_50.gif",
                       "wall_outlet_off_50.gif",
                       "xmas_on_50.gif",
                       "xmas_off_50.gif",
                       "confirm_30.gif"
                       };
//String filenames[] = {"teste1.txt"};

int used_memory = 0;


static final byte MARKER_BEGIN = 65;
static final byte MARKER_NEXT_FILE = 66;
static final byte MARKER_ARDUINO_RESET = 67;

final String serialPort = "COM4";


byte[] inBuffer = new byte[22]; 


void waitForArduino()
{
    println("Waiting for Arduino...");
    while (myPort.read() != MARKER_ARDUINO_RESET) { print("#"); delay(100); }; println();
    println("Arduino ready to go!");
}

void sendMaxStrings(int max_strings)
{
    println("Sending number of strings:" + max_strings);
    // Sends a marker to indicate the beginnng of a new file
    myPort.write(MARKER_BEGIN);
    myPort.write((byte)max_strings);
    while (myPort.available() == 0) { print("#"); delay(100); }; println();
    byte response = (byte)myPort.read();    
    if (response == max_strings)
        println("Number of strings received ok!");
    else
    {
        print("ERROR IN FILE SIZE, RECEIVED:");
        println(response);
        return;
    }   
}


int sendFile(String filename, int number)
{
    byte b[] = loadBytes(filename);
    println("->  File loaded:" + filename);
    println("->  File size sent:" + b.length);
    
    // Sends a marker to indicate the beginnng of a new file
    myPort.write(MARKER_NEXT_FILE);
    // Sending the file size
    int file_size = b.length;
    byte byte_lo = (byte)(file_size &0xFF);
    byte byte_hi = (byte)((file_size >> 8) &0xFF);      
    myPort.write(byte_lo);
    myPort.write(byte_hi);
    
    delay(100); while (myPort.available() <1) { print("#"); delay(100); }; // println();
    while (myPort.available() >0) print((char)myPort.read());
       
    println("->Sending file data... ");
    byte crc_byte = 0;
    int bar_length = 50;    
    float tick = b.length/(float)bar_length;
    print("0%");
    for (int j=0; j<bar_length-6; j++) print(" ");
    println("100%");
    int last_tick = 0;    
    for(int i = 0; i < b.length; i++) 
    {
        byte aux = b[i];
        myPort.write(b[i]);
        crc_byte = (byte) ((int)crc_byte^(int)aux);
        
        while(i/tick > last_tick)
        {
            print("#");
            last_tick++;
        }            
    }
    println();
    print("  File sent, CRC: " +  hex(crc_byte ));

    delay(100); while (myPort.available() <1) { print("#"); delay(100); }; println();
    while (myPort.available() >0) print((char)myPort.read());

    return  b.length; 
}

/*
int sendFile(String filename, int number)
{
    byte b[] = loadBytes(filename);
    print("File loaded:" + filename + "  size:" + b.length);
    
    // Sends a marker to indicate the beginnng of a new file
    myPort.write(MARKER_NEXT_FILE);
    // Sending the file size
    int file_size = b.length;
    byte byte_lo = (byte)(file_size &0xFF);
    byte byte_hi = (byte)((file_size >> 8) &0xFF);      
    myPort.write(byte_lo);
    myPort.write(byte_hi);
    
    delay(100); while (myPort.available() <2) { print("#"); delay(100); }; println();
    byte response_byte_lo = (byte)myPort.read();
    byte response_byte_hi = (byte)myPort.read();
    
    if (response_byte_lo != byte_lo || response_byte_hi != byte_hi)
    {
        print("ERROR IN FILE SIZE, RECEIVED:");
        println(response_byte_lo);
        return 0;
    }   
    println("Sending file data... ");
    byte crc_byte = 0;
    int bar_length = 50;    
    float tick = b.length/(float)bar_length;
    print("0%");
    for (int j=0; j<bar_length-6; j++) print(" ");
    println("100%");
    int last_tick = 0;    
    for(int i = 0; i < b.length; i++) 
    {
        byte aux = b[i];
        myPort.write(b[i]);
        crc_byte = (byte) ((int)crc_byte^(int)aux);
        
        while(i/tick > last_tick)
        {
            print("#");
            last_tick++;
        }            
    }

    print("  File sent!  ");
    delay(100); while (myPort.available() == 0) {print("#"); delay(100); };
    byte received_crc_byte = (byte) myPort.read();
    if (received_crc_byte == crc_byte)
        println("CRC received ok");
    else
        println("CRC FAILED!!!");
          
    return  b.length;  
}
/* */

void writeFlash()
{
    waitForArduino();
    sendMaxStrings(filenames.length);
    // Memory used for the header
    used_memory += 1 + 2*filenames.length;
    
    for (byte file_count = 0; file_count < filenames.length; file_count++)    
    {
         used_memory += sendFile(filenames[file_count], file_count);
         print("Used Memory:");
         println(used_memory);
    }
}

void checkFiles()
{
    int planned_memory = 1 + 2*filenames.length;

    for (byte file_count = 0; file_count < filenames.length; file_count++)    
    {
        byte b[] = loadBytes(filenames[file_count]);
        if (b!=null) planned_memory += b.length;
        print(b.length);
        print("\t");
        println(filenames[file_count]);
    }
    print("Planned Memory:");
    println(planned_memory);    
}

void setup() 
{
    myPort = new Serial(this, serialPort, 9600);  
  
    delay(100);
    myPort.clear();
    
    checkFiles();
}


void draw() 
{
    delay(100);
}

void keyPressed() 
{
    int keyIndex = -1;
    if (key == 'F' || key == 'f') 
    {
        writeFlash();
    }
}
